@@ -2,32 +2,39 @@
 # coding=utf-8
 # ------------------------------------------------------------------------
 
-from datetime import datetime
+from __future__ import absolute_import
 
+from datetime import datetime
+import logging
+import os
+import re
+
+# Try to import PIL in either of the two ways it can end up installed.
+try:
+    from PIL import Image
+except ImportError:
+    import Image
+
+from django import forms
+from django.conf import settings as django_settings
 from django.contrib import admin, messages
 from django.contrib.auth.decorators import permission_required
-from django.conf import settings as django_settings
+from django.contrib.sites.models import Site
+from django.core.urlresolvers import reverse
 from django.db import models
-from django.template.defaultfilters import filesizeformat
-from django.utils.safestring import mark_safe
-from django.utils import translation
-from django.utils.translation import ugettext_lazy as _
-from django.template.defaultfilters import slugify
 from django.http import HttpResponseRedirect
-# 1.2 from django.views.decorators.csrf import csrf_protect
+from django.shortcuts import render_to_response
+from django.template.context import RequestContext
+from django.template.defaultfilters import filesizeformat, slugify
+from django.utils.safestring import mark_safe
+from django.utils.translation import ungettext, ugettext_lazy as _
+from django.views.decorators.csrf import csrf_protect
 
 from feincms import settings
-from feincms.models import Base
-from feincms.utils import get_object
-
+from feincms.models import ExtensionsMixin
 from feincms.templatetags import feincms_thumbnail
-from feincms.translations import TranslatedObjectMixin, Translation, \
-    TranslatedObjectManager
-
-import re
-import os
-import logging
-from PIL import Image
+from feincms.translations import (TranslatedObjectMixin, Translation,
+    TranslatedObjectManager, admin_translationinline, lookup_translations)
 
 # ------------------------------------------------------------------------
 class CategoryManager(models.Manager):
@@ -71,31 +78,31 @@ class Category(models.Model):
 
         super(Category, self).save(*args, **kwargs)
 
+    def path_list(self):
+        if self.parent is None:
+            return [ self ]
+        p = self.parent.path_list()
+        p.append(self)
+        return p
+
+    def path(self):
+        return ' - '.join((f.title for f in self.path_list()))
 
 class CategoryAdmin(admin.ModelAdmin):
-    list_display      = ['parent', 'title']
+    list_display      = ['path']
     list_filter       = ['parent']
     list_per_page     = 25
     search_fields     = ['title']
     prepopulated_fields = { 'slug': ('title',), }
 
-
 # ------------------------------------------------------------------------
-class MediaFileBase(Base, TranslatedObjectMixin):
+class MediaFileBase(models.Model, ExtensionsMixin, TranslatedObjectMixin):
     """
-    Abstract media file class. Inherits from :class:`feincms.module.Base`
+    Abstract media file class. Includes the :class:`feincms.models.ExtensionsMixin`
     because of the (handy) extension mechanism.
     """
 
-    from django.core.files.storage import FileSystemStorage
-    default_storage_class = getattr(django_settings, 'DEFAULT_FILE_STORAGE',
-                                    'django.core.files.storage.FileSystemStorage')
-    default_storage = get_object(default_storage_class)
-
-    fs = default_storage(location=settings.FEINCMS_MEDIALIBRARY_ROOT,
-                           base_url=settings.FEINCMS_MEDIALIBRARY_URL)
-
-    file = models.FileField(_('file'), max_length=255, upload_to=settings.FEINCMS_MEDIALIBRARY_UPLOAD_TO, storage=fs)
+    file = models.FileField(_('file'), max_length=255, upload_to=settings.FEINCMS_MEDIALIBRARY_UPLOAD_TO)
     type = models.CharField(_('file type'), max_length=12, editable=False, choices=())
     created = models.DateTimeField(_('created'), editable=False, default=datetime.now)
     copyright = models.CharField(_('copyright'), max_length=200, blank=True)
@@ -107,6 +114,7 @@ class MediaFileBase(Base, TranslatedObjectMixin):
 
     class Meta:
         abstract = True
+        ordering = ['-created']
         verbose_name = _('media file')
         verbose_name_plural = _('media files')
 
@@ -121,7 +129,7 @@ class MediaFileBase(Base, TranslatedObjectMixin):
     formatted_file_size.admin_order_field = 'file_size'
 
     def formatted_created(self):
-        return self.created.strftime("%Y-%m-%d %H:%M")
+        return self.created.strftime("%Y-%m-%d")
     formatted_created.short_description = _("created")
     formatted_created.admin_order_field = 'created'
 
@@ -146,27 +154,24 @@ class MediaFileBase(Base, TranslatedObjectMixin):
 
     def __init__(self, *args, **kwargs):
         super(MediaFileBase, self).__init__(*args, **kwargs)
-        if self.file and self.file.path:
-            self._original_file_path = self.file.path
+        if self.file:
+            self._original_file_name = self.file.name
 
     def __unicode__(self):
         trans = None
 
-        # This might be provided using a .extra() clause to avoid hundreds of extra queries:
-        if hasattr(self, "preferred_translation"):
-            trans = getattr(self, "preferred_translation", u"")
-        else:
-            try:
-                trans = unicode(self.translation)
-            except models.ObjectDoesNotExist:
-                pass
-            except AttributeError, e:
-                pass
+        try:
+            trans = self.translation
+        except models.ObjectDoesNotExist:
+            pass
+        except AttributeError:
+            pass
 
         if trans:
-            return trans
-        else:
-            return os.path.basename(self.file.name)
+            trans = unicode(trans)
+            if trans.strip():
+                return trans
+        return os.path.basename(self.file.name)
 
     def get_absolute_url(self):
         return self.file.url
@@ -174,12 +179,17 @@ class MediaFileBase(Base, TranslatedObjectMixin):
     def file_type(self):
         t = self.filetypes_dict[self.type]
         if self.type == 'image':
+            # get_image_dimensions is expensive / slow if the storage is not local filesystem (indicated by availability the path property)
+            try:
+                self.file.path
+            except NotImplementedError:
+                return t
             try:
                 from django.core.files.images import get_image_dimensions
                 d = get_image_dimensions(self.file.file)
-                if d: t += "<br/>%d&times;%d" % ( d[0], d[1] )
-            except IOError, e:
-                t += "<br/>(%s)" % e.strerror
+                if d: t += " %d&times;%d" % ( d[0], d[1] )
+            except (IOError, ValueError), e:
+                t += " (%s)" % e.strerror
         return t
     file_type.admin_order_field = 'type'
     file_type.short_description = _('file type')
@@ -193,12 +203,16 @@ class MediaFileBase(Base, TranslatedObjectMixin):
         the file name later on, this can be used to access the file name from
         JS, like for example a TinyMCE connector shim.
         """
-        from os.path import basename
         from feincms.utils import shorten_string
-        return u'<input type="hidden" class="medialibrary_file_path" name="_media_path_%d" value="%s" /> %s' % (
+        return u'<input type="hidden" class="medialibrary_file_path" name="_media_path_%d" value="%s" id="_refkey_%d" /> %s <br />%s, %s' % (
                 self.id,
                 self.file.name,
-                shorten_string(basename(self.file.name), max_length=28), )
+                self.id,
+                shorten_string(os.path.basename(self.file.name), max_length=40),
+                self.file_type(),
+                self.formatted_file_size(),
+                )
+    file_info.admin_order_field = 'file'
     file_info.short_description = _('file info')
     file_info.allow_tags = True
 
@@ -266,12 +280,9 @@ class MediaFileBase(Base, TranslatedObjectMixin):
             except (OSError, IOError), e:
                 self.type = self.determine_file_type('***') # It's binary something
 
-        if getattr(self, '_original_file_path', None):
-            if self.file.path != self._original_file_path:
-                try:
-                    os.unlink(self._original_file_path)
-                except:
-                    pass
+        if getattr(self, '_original_file_name', None):
+            if self.file.name != self._original_file_name:
+                self.file.storage.delete(self._original_file_name)
 
         super(MediaFileBase, self).save(*args, **kwargs)
         self.purge_translation_cache()
@@ -317,16 +328,11 @@ class MediaFileTranslation(Translation(MediaFile)):
         return self.caption
 
 #-------------------------------------------------------------------------
-class MediaFileTranslationInline(admin.StackedInline):
-    model   = MediaFileTranslation
-    max_num = len(django_settings.LANGUAGES)
-
-
 def admin_thumbnail(obj):
     if obj.type == 'image':
         image = None
         try:
-            image = feincms_thumbnail.thumbnail(obj.file.name, '100x60')
+            image = feincms_thumbnail.thumbnail(obj.file.name, '100x100')
         except:
             pass
 
@@ -342,14 +348,95 @@ admin_thumbnail.short_description = _('Preview')
 admin_thumbnail.allow_tags = True
 
 #-------------------------------------------------------------------------
+def assign_category(modeladmin, request, queryset):
+    class AddCategoryForm(forms.Form):
+        _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
+        category = forms.ModelChoiceField(Category.objects.all())
+
+    form = None
+    if 'apply' in request.POST:
+        form = AddCategoryForm(request.POST)
+        if form.is_valid():
+            category = form.cleaned_data['category']
+
+            count = 0
+            for mediafile in queryset:
+                category.mediafile_set.add(mediafile)
+                count += 1
+
+            message = ungettext('Successfully added %(count)d media file to %(category)s.',
+                                'Successfully added %(count)d media files to %(category)s.',
+                                count) % {'count':count, 'category':category}
+            modeladmin.message_user(request, message)
+            return HttpResponseRedirect(request.get_full_path())
+    if 'cancel' in request.POST:
+        return HttpResponseRedirect(request.get_full_path())
+
+    if not form:
+        form = AddCategoryForm(initial={
+            '_selected_action': request.POST.getlist(admin.ACTION_CHECKBOX_NAME),
+            })
+
+    return render_to_response('admin/medialibrary/add_to_category.html', {
+        'mediafiles': queryset,
+        'category_form': form,
+        }, context_instance=RequestContext(request))
+
+assign_category.short_description = _('Add selected media files to category')
+
+#-------------------------------------------------------------------------
+def save_as_zipfile(modeladmin, request, queryset):
+    from .zip import export_zipfile
+
+    site = Site.objects.get_current()
+    try:
+        zip_name = export_zipfile(site, queryset)
+        messages.info(request, _("ZIP file exported as %s") % zip_name)
+    except Exception, e:
+        messages.error(request, _("ZIP file export failed: %s") % str(e))
+
+    return HttpResponseRedirect(os.path.join(django_settings.MEDIA_URL, zip_name))
+
+save_as_zipfile.short_description = _('Export selected media files as zip file')
+
+# ------------------------------------------------------------------------
+class MediaFileAdminForm(forms.ModelForm):
+    class Meta:
+        model = MediaFile
+
+    def __init__(self, *args, **kwargs):
+        super(MediaFileAdminForm, self).__init__(*args, **kwargs)
+        if settings.FEINCMS_MEDIAFILE_OVERWRITE and self.instance.id:
+            self.original_name = self.instance.file.name
+
+            def gen_fname(instance, filename):
+                self.instance.file.storage.delete(self.original_name)
+                return self.original_name
+            self.instance.file.field.generate_filename = gen_fname
+
+    def clean_file(self):
+        if settings.FEINCMS_MEDIAFILE_OVERWRITE and hasattr(self, 'original_name'):
+            new_base, new_ext = os.path.splitext(self.cleaned_data['file'].name)
+            old_base, old_ext = os.path.splitext(self.original_name)
+
+            if new_ext.lower() != old_ext.lower():
+                raise forms.ValidationError(_("Cannot overwrite with different file type (attempt to overwrite a %(old_ext)s with a %(new_ext)s)") % { 'old_ext': old_ext, 'new_ext': new_ext })
+
+        return self.cleaned_data['file']
+
+# -----------------------------------------------------------------------
 class MediaFileAdmin(admin.ModelAdmin):
+    save_on_top       = True
+    form              = MediaFileAdminForm
     date_hierarchy    = 'created'
-    inlines           = [MediaFileTranslationInline]
-    list_display      = ['__unicode__', admin_thumbnail, 'file_type', 'copyright', 'file_info', 'formatted_file_size', 'formatted_created']
+    inlines           = [admin_translationinline(MediaFileTranslation)]
+    list_display      = [admin_thumbnail, '__unicode__', 'file_info', 'formatted_created']
+    list_display_links = ['__unicode__']
     list_filter       = ['type', 'categories']
     list_per_page     = 25
     search_fields     = ['copyright', 'file', 'translations__caption']
     filter_horizontal = ("categories",)
+    actions           = [assign_category, save_as_zipfile]
 
     def get_urls(self):
         from django.conf.urls.defaults import url, patterns
@@ -364,83 +451,28 @@ class MediaFileAdmin(admin.ModelAdmin):
     def changelist_view(self, request, extra_context=None):
         if extra_context is None:
             extra_context = {}
-        extra_context['categories'] = Category.objects.all()
+        extra_context['categories'] = Category.objects.order_by('title')
         return super(MediaFileAdmin, self).changelist_view(request, extra_context=extra_context)
 
     @staticmethod
-    # 1.2 @csrf_protect
+    @csrf_protect
     @permission_required('medialibrary.add_mediafile')
     def bulk_upload(request):
-        from django.core.urlresolvers import reverse
-        from django.utils.functional import lazy
-
-        def import_zipfile(request, category_id, data):
-            import zipfile
-            from os import path
-
-            category = None
-            if category_id:
-                category = Category.objects.get(pk=int(category_id))
-
-            try:
-                z = zipfile.ZipFile(data)
-
-                storage = MediaFile.fs
-                if not storage:
-                    messages.error(request, _("Could not access storage"))
-                    return
-
-                count = 0
-                for zi in z.infolist():
-                    if not zi.filename.endswith('/'):
-                        from django.template.defaultfilters import slugify
-                        from django.core.files.base import ContentFile
-
-                        bname = path.basename(zi.filename)
-                        if bname and not bname.startswith(".") and "." in bname:
-                            fname, ext = path.splitext(bname)
-                            target_fname = slugify(fname) + ext.lower()
-
-                            mf = MediaFile()
-                            mf.file.save(target_fname, ContentFile(z.read(zi.filename)))
-                            mf.save()
-                            if category:
-                                mf.categories.add(category)
-                            count += 1
-
-                messages.info(request, _("%d files imported") % count)
-            except Exception, e:
-                messages.error(request, _("ZIP file invalid: %s") % str(e))
-                return
+        from .zip import import_zipfile
 
         if request.method == 'POST' and 'data' in request.FILES:
-            import_zipfile(request, request.POST.get('category'), request.FILES['data'])
+            try:
+                count = import_zipfile(request.POST.get('category'), request.POST.get('overwrite', False), request.FILES['data'])
+                messages.info(request, _("%d files imported") % count)
+            except Exception, e:
+                messages.error(request, _("ZIP import failed: %s") % str(e))
         else:
             messages.error(request, _("No input file given"))
 
         return HttpResponseRedirect(reverse('admin:medialibrary_mediafile_changelist'))
 
     def queryset(self, request):
-        qs = super(MediaFileAdmin, self).queryset(request)
-
-        # FIXME: This is an ugly hack but it avoids 1-3 queries per *FILE*
-        # retrieving the translation information
-        if django_settings.DATABASE_ENGINE == 'postgresql_psycopg2':
-            qs = qs.extra(
-                select = {
-                    'preferred_translation':
-                        """SELECT caption FROM medialibrary_mediafiletranslation
-                        WHERE medialibrary_mediafiletranslation.parent_id = medialibrary_mediafile.id
-                        ORDER BY
-                            language_code = %s DESC,
-                            language_code = %s DESC,
-                            LENGTH(language_code) DESC
-                        LIMIT 1
-                        """
-                },
-                select_params = (translation.get_language(), django_settings.LANGUAGE_CODE)
-            )
-        return qs
+        return super(MediaFileAdmin, self).queryset(request).transform(lookup_translations())
 
     def save_model(self, request, obj, form, change):
         obj.purge_translation_cache()
@@ -448,4 +480,3 @@ class MediaFileAdmin(admin.ModelAdmin):
 
 
 #-------------------------------------------------------------------------
-
